@@ -67,6 +67,8 @@ struct _GUPnPContextPrivate {
 
         SoupServer  *server; /* Started on demand */
         char        *server_url;
+
+        GList       *host_path_datas;
 };
 
 enum {
@@ -77,9 +79,17 @@ enum {
         PROP_SUBSCRIPTION_TIMEOUT
 };
 
+typedef struct {
+        char *local_path;
+
+        GRegex *regex;
+} UserAgent;
+
 typedef struct {
         char *local_path;
         char *server_path;
+
+        GList *user_agents;
 } HostPathData;
 
 /*
@@ -118,12 +128,13 @@ gupnp_context_constructor (GType                  type,
                            guint                  n_props,
                            GObjectConstructParam *props)
 {
-	GObject *object;
-	GUPnPContext *context;
+        GObject *object;
+        GUPnPContext *context;
+        char *user_agent;
 
-	object = G_OBJECT_CLASS (gupnp_context_parent_class)->constructor
+        object = G_OBJECT_CLASS (gupnp_context_parent_class)->constructor
                 (type, n_props, props);
-	context = GUPNP_CONTEXT (object);
+        context = GUPNP_CONTEXT (object);
 
         context->priv->session = soup_session_async_new_with_options
                 (SOUP_SESSION_IDLE_TIMEOUT,
@@ -132,13 +143,22 @@ gupnp_context_constructor (GType                  type,
                  gssdp_client_get_main_context (GSSDP_CLIENT (context)),
                  NULL);
 
-	if (g_getenv ("GUPNP_DEBUG")) {
-		SoupLogger *logger;
-		logger = soup_logger_new (SOUP_LOGGER_LOG_BODY, -1);
-		soup_logger_attach (logger, context->priv->session);
-	}
+        user_agent = g_strdup_printf ("%s GUPnP/" VERSION " DLNADOC/1.50",
+                                      g_get_application_name ()? : "");
+        g_object_set (context->priv->session,
+                      SOUP_SESSION_USER_AGENT,
+                      user_agent,
+                      NULL);
+        g_free (user_agent);
 
-	return object;
+        if (g_getenv ("GUPNP_DEBUG")) {
+                SoupLogger *logger;
+                logger = soup_logger_new (SOUP_LOGGER_LOG_BODY, -1);
+                soup_session_add_feature (context->priv->session,
+                                          SOUP_SESSION_FEATURE (logger));
+        }
+
+        return object;
 }
 
 static void
@@ -216,6 +236,14 @@ gupnp_context_dispose (GObject *object)
                 context->priv->server = NULL;
         }
 
+        while (context->priv->host_path_datas) {
+                HostPathData *data;
+
+                data = (HostPathData *) context->priv->host_path_datas->data;
+
+                gupnp_context_unhost_path (context, data->server_path);
+        }
+
         /* Call super */
         object_class = G_OBJECT_CLASS (gupnp_context_parent_class);
         object_class->dispose (object);
@@ -252,7 +280,7 @@ gupnp_context_class_init (GUPnPContextClass *klass)
         g_type_class_add_private (klass, sizeof (GUPnPContextPrivate));
 
         /**
-         * GUPnPContext:port
+         * GUPnPContext:port:
          *
          * The port to run on. Set to 0 if you don't care what port to run on.
          **/
@@ -270,7 +298,7 @@ gupnp_context_class_init (GUPnPContextClass *klass)
                                     G_PARAM_STATIC_BLURB));
 
         /**
-         * GUPnPContext:server
+         * GUPnPContext:server:
          *
          * The #SoupServer HTTP server used by GUPnP.
          **/
@@ -287,7 +315,7 @@ gupnp_context_class_init (GUPnPContextClass *klass)
                                       G_PARAM_STATIC_BLURB));
 
         /**
-         * GUPnPContext:session
+         * GUPnPContext:session:
          *
          * The #SoupSession object used by GUPnP.
          **/
@@ -304,7 +332,7 @@ gupnp_context_class_init (GUPnPContextClass *klass)
                                       G_PARAM_STATIC_BLURB));
 
         /**
-         * GUPnPContext:subscription-timeout
+         * GUPnPContext:subscription-timeout:
          *
          * The preferred subscription timeout: the number of seconds after
          * which subscriptions are renewed. Set to '0' if subscriptions 
@@ -327,13 +355,13 @@ gupnp_context_class_init (GUPnPContextClass *klass)
 }
 
 /**
- * gupnp_context_get_session
+ * gupnp_context_get_session:
  * @context: A #GUPnPContext
  *
  * Get the #SoupSession object that GUPnP is using.
  *
- * Return value: The #SoupSession used by GUPnP. Do not unref this when
- * finished.
+ * Return value: (transfer none): The #SoupSession used by GUPnP. Do not unref
+ * this when finished.
  **/
 SoupSession *
 gupnp_context_get_session (GUPnPContext *context)
@@ -358,7 +386,7 @@ default_server_handler (SoupServer        *server,
 }
 
 /**
- * gupnp_context_get_server
+ * gupnp_context_get_server:
  * @context: A #GUPnPContext
  *
  * Get the #SoupServer HTTP server that GUPnP is using.
@@ -371,12 +399,21 @@ gupnp_context_get_server (GUPnPContext *context)
         g_return_val_if_fail (GUPNP_IS_CONTEXT (context), NULL);
 
         if (context->priv->server == NULL) {
+                const char *ip;
+
+                ip =  gssdp_client_get_host_ip (GSSDP_CLIENT (context));
+                SoupAddress *addr = soup_address_new (ip, context->priv->port);
+                soup_address_resolve_sync (addr, NULL);
+
                 context->priv->server = soup_server_new
                         (SOUP_SERVER_PORT,
                          context->priv->port,
                          SOUP_SERVER_ASYNC_CONTEXT,
                          gssdp_client_get_main_context (GSSDP_CLIENT (context)),
+                         SOUP_SERVER_INTERFACE,
+                         addr,
                          NULL);
+                g_object_unref (addr);
 
                 soup_server_add_handler (context->priv->server, NULL,
                                          default_server_handler, context,
@@ -418,7 +455,7 @@ _gupnp_context_get_server_url (GUPnPContext *context)
 }
 
 /**
- * gupnp_context_new
+ * gupnp_context_new:
  * @main_context: A #GMainContext, or %NULL to use the default one
  * @interface: The network interface to use, or %NULL to auto-detect.
  * @port: Port to run on, or 0 if you don't care what port is used.
@@ -444,7 +481,7 @@ gupnp_context_new (GMainContext *main_context,
 }
 
 /**
- * gupnp_context_get_host_ip
+ * gupnp_context_get_host_ip:
  * @context: A #GUPnPContext
  *
  * Get the IP address we advertise ourselves as using.
@@ -462,7 +499,7 @@ gupnp_context_get_host_ip (GUPnPContext *context)
 }
 
 /**
- * gupnp_context_get_port
+ * gupnp_context_get_port:
  * @context: A #GUPnPContext
  *
  * Get the port that the SOAP server is running on.
@@ -481,7 +518,7 @@ gupnp_context_get_port (GUPnPContext *context)
 }
 
 /**
- * gupnp_context_set_subscription_timeout
+ * gupnp_context_set_subscription_timeout:
  * @context: A #GUPnPContext
  * @timeout: Event subscription timeout in seconds
  *
@@ -501,7 +538,7 @@ gupnp_context_set_subscription_timeout (GUPnPContext *context,
 }
 
 /**
- * gupnp_context_get_subscription_timeout
+ * gupnp_context_get_subscription_timeout:
  * @context: A #GUPnPContext
  *
  * Get the event subscription timeout (in seconds), or 0 meaning there is no
@@ -521,18 +558,44 @@ gupnp_context_get_subscription_timeout (GUPnPContext *context)
  * if any to make sure we append the locale suffix in a canonical way. */
 static char *
 construct_local_path (const char   *requested_path,
+                      const char   *user_agent,
                       HostPathData *host_path_data)
 {
         GString *str;
+        char *local_path;
         int len;
 
+        local_path = NULL;
+
+        if (user_agent != NULL) {
+                GList *node;
+
+                for (node = host_path_data->user_agents;
+                     node;
+                     node = node->next) {
+                        UserAgent *agent;
+
+                        agent = node->data;
+
+                        if (g_regex_match (agent->regex,
+                                           user_agent,
+                                           0,
+                                           NULL)) {
+                                local_path = agent->local_path;
+                        }
+                }
+        }
+
+        if (local_path == NULL)
+                local_path = host_path_data->local_path;
+
         if (!requested_path || *requested_path == 0)
-                return g_strdup (host_path_data->local_path);
+                return g_strdup (local_path);
 
         if (*requested_path != '/')
                 return NULL; /* Absolute paths only */
 
-        str = g_string_new (host_path_data->local_path);
+        str = g_string_new (local_path);
 
         /* Skip the length of the path relative to which @requested_path
          * is specified. */
@@ -596,11 +659,14 @@ host_path_handler (SoupServer        *server,
         GList *locales, *orig_locales;
         GMappedFile *mapped_file;
         GError *error;
+        HostPathData *host_path_data;
+        const char *user_agent;
 
         orig_locales = NULL;
         locales      = NULL;
         local_path   = NULL;
         path_to_open = NULL;
+        host_path_data = (HostPathData *) user_data;
 
         if (msg->method != SOUP_METHOD_GET &&
             msg->method != SOUP_METHOD_HEAD) {
@@ -609,8 +675,11 @@ host_path_handler (SoupServer        *server,
                 goto DONE;
         }
 
+        user_agent = soup_message_headers_get_one (msg->request_headers,
+                                                   "User-Agent");
+
         /* Construct base local path */
-        local_path = construct_local_path (path, (HostPathData *) user_data);
+        local_path = construct_local_path (path, user_agent, host_path_data);
         if (!local_path) {
                 soup_message_set_status (msg, SOUP_STATUS_BAD_REQUEST);
 
@@ -706,10 +775,9 @@ host_path_handler (SoupServer        *server,
                         goto DONE;
                 }
 
-                if (have_range && (length < 0                   ||
-                                   offset < 0                   ||
-                                   length > st.st_size - offset ||
-                                   offset >= st.st_size)) {
+                if (have_range && (length > st.st_size - offset ||
+                                   st.st_size < 0 ||
+                                   (off_t) offset >= st.st_size)) {
                         soup_message_set_status
                                 (msg,
                                  SOUP_STATUS_REQUESTED_RANGE_NOT_SATISFIABLE);
@@ -722,7 +790,7 @@ host_path_handler (SoupServer        *server,
                              (g_mapped_file_get_contents (mapped_file) + offset,
                               length,
                               mapped_file,
-                              (GDestroyNotify) g_mapped_file_free);
+                              (GDestroyNotify) g_mapped_file_unref);
 
                 soup_message_body_append_buffer (msg->response_body, buffer);
 
@@ -742,7 +810,7 @@ host_path_handler (SoupServer        *server,
                 char *length;
 
                 length = g_strdup_printf ("%lu", (gulong) st.st_size);
-	        soup_message_headers_append (msg->response_headers,
+                soup_message_headers_append (msg->response_headers,
                                              "Content-Length",
                                              length);
                 g_free (length);
@@ -784,13 +852,36 @@ host_path_handler (SoupServer        *server,
         }
 }
 
-HostPathData *
+static UserAgent *
+user_agent_new (const char *local_path,
+                GRegex     *regex)
+{
+        UserAgent *agent;
+
+        agent = g_slice_new0 (UserAgent);
+
+        agent->local_path = g_strdup (local_path);
+        agent->regex = g_regex_ref (regex);
+
+        return agent;
+}
+
+static void
+user_agent_free (UserAgent *agent)
+{
+        g_free (agent->local_path);
+        g_regex_unref (agent->regex);
+
+        g_slice_free (UserAgent, agent);
+}
+
+static HostPathData *
 host_path_data_new (const char *local_path,
                     const char *server_path)
 {
         HostPathData *path_data;
 
-        path_data = g_slice_new (HostPathData);
+        path_data = g_slice_new0 (HostPathData);
 
         path_data->local_path  = g_strdup (local_path);
         path_data->server_path = g_strdup (server_path);
@@ -798,17 +889,28 @@ host_path_data_new (const char *local_path,
         return path_data;
 }
 
-void
+static void
 host_path_data_free (HostPathData *path_data)
 {
         g_free (path_data->local_path);
         g_free (path_data->server_path);
+        while (path_data->user_agents) {
+                UserAgent *agent;
+
+                agent = path_data->user_agents->data;
+
+                user_agent_free (agent);
+
+                path_data->user_agents = g_list_delete_link (
+                                path_data->user_agents,
+                                path_data->user_agents);
+        }
 
         g_slice_free (HostPathData, path_data);
 }
 
 /**
- * gupnp_context_host_path
+ * gupnp_context_host_path:
  * @context: A #GUPnPContext
  * @local_path: Path to the local file or folder to be hosted
  * @server_path: Web server path where @local_path should be hosted
@@ -831,17 +933,72 @@ gupnp_context_host_path (GUPnPContext *context,
 
         server = gupnp_context_get_server (context);
 
-        path_data = host_path_data_new (local_path, server_path);
+        path_data = host_path_data_new (local_path,
+                                        server_path);
 
         soup_server_add_handler (server,
                                  server_path,
                                  host_path_handler,
                                  path_data,
-                                 (GDestroyNotify) host_path_data_free);
+                                 NULL);
+
+        context->priv->host_path_datas =
+                g_list_append (context->priv->host_path_datas,
+                               path_data);
+}
+
+static unsigned int
+path_compare_func (HostPathData *path_data,
+                   const char   *server_path)
+{
+        return strcmp (path_data->server_path, server_path);
 }
 
 /**
- * gupnp_context_unhost_path
+ * gupnp_context_host_path_for_agent:
+ * @context: A #GUPnPContext
+ * @local_path: Path to the local file or folder to be hosted
+ * @server_path: Web server path already being hosted
+ * @user_agent: The user-agent as a #GRegex.
+ *
+ * Use this method to serve different local path to specific user-agent(s). The
+ * path @server_path must already be hosted by @context.
+ *
+ * Return value: %TRUE on success, %FALSE otherwise.
+ **/
+gboolean
+gupnp_context_host_path_for_agent (GUPnPContext *context,
+                                   const char   *local_path,
+                                   const char   *server_path,
+                                   GRegex       *user_agent)
+{
+        GList *node;
+
+        g_return_val_if_fail (GUPNP_IS_CONTEXT (context), FALSE);
+        g_return_val_if_fail (local_path != NULL, FALSE);
+        g_return_val_if_fail (server_path != NULL, FALSE);
+        g_return_val_if_fail (user_agent != NULL, FALSE);
+
+        node = g_list_find_custom (context->priv->host_path_datas,
+                                   server_path,
+                                   (GCompareFunc) path_compare_func);
+        if (node != NULL) {
+                HostPathData *path_data;
+                UserAgent *agent;
+
+                path_data = (HostPathData *) node->data;
+                agent = user_agent_new (local_path, user_agent);
+
+                path_data->user_agents = g_list_append (path_data->user_agents,
+                                                        agent);
+
+                return TRUE;
+        } else
+                return FALSE;
+}
+
+/**
+ * gupnp_context_unhost_path:
  * @context: A #GUPnPContext
  * @server_path: Web server path where the file or folder is hosted
  *
@@ -852,11 +1009,24 @@ gupnp_context_unhost_path (GUPnPContext *context,
                            const char   *server_path)
 {
         SoupServer *server;
+        HostPathData *path_data;
+        GList *node;
 
         g_return_if_fail (GUPNP_IS_CONTEXT (context));
         g_return_if_fail (server_path != NULL);
 
         server = gupnp_context_get_server (context);
 
+        node = g_list_find_custom (context->priv->host_path_datas,
+                                   server_path,
+                                   (GCompareFunc) path_compare_func);
+        g_return_if_fail (node != NULL);
+
+        path_data = (HostPathData *) node->data;
+        context->priv->host_path_datas = g_list_delete_link (
+                        context->priv->host_path_datas,
+                        node);
+
         soup_server_remove_handler (server, server_path);
+        host_path_data_free (path_data);
 }
