@@ -42,15 +42,13 @@ static GUPnPDeviceInfo *
 gupnp_device_proxy_get_device (GUPnPDeviceInfo *info,
                                xmlNode         *element)
 {
-        GUPnPDeviceProxy     *proxy, *device;
+        GUPnPDeviceProxy     *device;
         GUPnPResourceFactory *factory;
         GUPnPContext         *context;
         GUPnPXMLDoc          *doc;
         const char           *location;
         const SoupURI        *url_base;
 
-        proxy = GUPNP_DEVICE_PROXY (info);
-
         factory = gupnp_device_info_get_resource_factory (info);
         context = gupnp_device_info_get_context (info);
         doc = _gupnp_device_info_get_document (info);
@@ -72,7 +70,6 @@ static GUPnPServiceInfo *
 gupnp_device_proxy_get_service (GUPnPDeviceInfo *info,
                                 xmlNode         *element)
 {
-        GUPnPDeviceProxy     *proxy;
         GUPnPResourceFactory *factory;
         GUPnPServiceProxy    *service;
         GUPnPContext         *context;
@@ -80,8 +77,6 @@ gupnp_device_proxy_get_service (GUPnPDeviceInfo *info,
         const char           *location, *udn;
         const SoupURI        *url_base;
 
-        proxy = GUPNP_DEVICE_PROXY (info);
-
         factory = gupnp_device_info_get_resource_factory (info);
         context = gupnp_device_info_get_context (info);
         doc = _gupnp_device_info_get_document (info);
